<?php

namespace App\Http\Controllers\API_MOBILE\Chambre;

use App\Http\Controllers\Controller;
use App\Http\Resources\ChambreResource;
use App\Models\Chambre;
use App\Models\Entreprise;
use Exception;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\ValidationException;

class ChambreController_MOBILE extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        try {
            $Chambre = Chambre::with(["entreprise:id,name,ICE","zone:id,name"])->paginate(20);

            return ChambreResource::collection($Chambre);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                "Name" => "required|string",
                "EntrepriseICE" => "required|integer",
                "ZoneId"=>"required|integer" ,
                "Surface"=>"required|numeric",
                "Temperature"=>"required|numeric"
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            $entreprise = Entreprise::where('ICE', $request->input('EntrepriseICE'))->first();

            if (!$entreprise) {
                throw new Exception('Enterprise not found');
            }

            DB::beginTransaction();

            $chambre = new Chambre();
            $chambre->Name = $request->input('Name');
            $chambre->EntrepriseId = $entreprise->id ;
            $chambre->ZoneId = $request->input("ZoneId") ;
            $chambre->Surface = $request->input("Surface");
            $chambre->Temperature = $request->input("Temperature");
            $chambre->save();

            DB::commit();

            return response()->json(["message" => "Chambre created successfully!", "status" => 201]);
        } catch (ValidationException $e) {
            DB::rollBack();
            return response()->json(["errors" => $e->errors(), "status" => 422]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        try {

            $Chambre = Chambre::where("EntrepriseId",$id)->with(["entreprise:id,name,ICE","zone:id,name"])->paginate(20);

            return ChambreResource::collection($Chambre);
        } catch (ModelNotFoundException $e) {
            return response()->json(["error" => "Chambre not found", "status" => 404]);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
        try {

            $zone = Chambre::findOrFail($id);

            $validator = Validator::make($request->all(), [
                "Name" => "string",
                "ZoneId"=>"integer" ,
                "Surface"=>"numeric",
                "Temperature"=>"numeric"
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            DB::beginTransaction();

            $zone->fill($request->all());
            $zone->save();

            DB::commit();

            return response()->json(["message" => "Chambre updated successfully", "status" => 200]);
        } catch (ValidationException $e) {
            DB::rollBack();
            return response()->json(["errors" => $e->errors(), "status" => 422]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        try {

            $zone = Chambre::findOrFail($id);

            DB::beginTransaction();

            $zone->forceDelete();

            DB::commit();

            return response()->json(["message" => "Chambre deleted successfully", "status" => 200]);
        } catch (ModelNotFoundException $e) {
            DB::rollBack();
            return response()->json(["error" => "Chambre not found", "status" => 404]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }
}
