<?php

namespace App\Http\Controllers\API_MOBILE\Nutrition;

use App\Http\Controllers\Controller;
use App\Http\Resources\NutritionResource;
use App\Models\Entreprise;
use App\Models\Nutrition;
use Exception;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\ValidationException;
use Illuminate\Support\Str;

class NutritionController_MOBILE extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        try {
            $Nutrition = Nutrition::paginate(20);
            return NutritionResource::collection($Nutrition);
    }catch(Exception $e){
        return response()->json(["error" => $e->getMessage(), "status" => 500]);
    }
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {

        try {
            $validator = Validator::make($request->all(), [
                "Name" => "required|string",
                "Image" => "nullable|string", // Assuming the image is sent as base64 string
                "EntrepriseICE" => "required|integer|exists:entreprises,ICE",
                "Description"=>"string|max:255",
                "ChambreID" =>"required|integer|exists:chambres,id",
                "FournisseurID"=>"required|integer|exists:fournisseurs,id",
                "BarCode"=>"required|integer" ,
                "DLCDate"=>"required|date",
                "DDMDate"=>"required|date",
                "ExpirationAlert" => "required|integer|in:0,1",
                "Category"=>"string|max:255"
            ]);


            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            $entreprise = Entreprise::where('ICE', $request->input('EntrepriseICE'))->first();
            //$chambre = Chambre::where('id', $request->input('ChambreID'))->first();
            //$fournisseur = Fournisseur::where('id', $request->input('FournisseurID'))->first();


            DB::beginTransaction();

            $nutrittion = new Nutrition();
            $nutrittion->Name = $request->input('Name');
            $nutrittion->EntrepriseId = $entreprise->id ;
            $nutrittion->ChambreId = $request->input("ChambreID") ;
            $nutrittion->FournisseurId = $request->input("FournisseurID") ;
            $nutrittion->BarCode = $request->input("BarCode") ;
            $nutrittion->DLCDate = $request->input("DLCDate") ;
            $nutrittion->DDMDate = $request->input("DDMDate") ;
            $nutrittion->ExpirationAlert = $request->input("ExpirationAlert") ;
            $nutrittion->Category = $request->input("Category") ;
            $nutrittion->Description = $request->input("Description");

            if ($request->has("Image")) {
                // Extract the image extension from the base64 string
                $extension = explode('/', mime_content_type($request->input("Image")))[1];

                // Decode the base64 image string
                $imageData = base64_decode(preg_replace('#^data:image/\w+;base64,#i', '', $request->input("Image")));

                // Generate a unique filename based on the current timestamp
                $imageName = time() . '_' . Str::random(10) . '.' . $extension;


                Storage::disk('public')->put('NutritionImages/' . $imageName, $imageData);


                $nutrittion->Image = 'NutritionImages/' . $imageName;


                $nutrittion
                ->addMedia(storage_path('app/public/NutritionImages/' . $imageName))
                ->usingFileName($imageName)
                ->toMediaCollection("NutritionImages" , "NutritionImages");
            }

            $nutrittion->save();

            DB::commit();

            return response()->json(["message" => "Nutrition created successfully!", "status" => 201]);
        } catch (ValidationException $e) {
            DB::rollBack();
            return response()->json(["errors" => $e->errors(), "status" => 422]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {

        //dd($id);
        try {

            $Nutrition = Nutrition::where("EntrepriseId",$id)->with(["entreprise:id,name,ICE","chambre:id,name","fournisseur:id,name"])->paginate(20);
            return NutritionResource::collection($Nutrition);
        } catch (ModelNotFoundException $e) {
            return response()->json(["error" => "Nutrition not found", "status" => 404]);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {

        try {

            $Nutrition = Nutrition::findOrFail($id);

            $validator = Validator::make($request->all(),[
                "Name" => "string",
                "Image" => "nullable|string", // Assuming the image is sent as base64 string
                "Description"=>"string|max:255",
                "ChambreID" =>"integer|exists:chambres,id",
                "FournisseurID"=>"integer|exists:fournisseurs,id",
                "BarCode"=>"required|integer" ,
                "DLCDate"=>"required|date",
                "DDMDate"=>"required|date",
                "ExpirationAlert" => "required|integer|in:0,1",
                "Category"=>"string|max:255"

            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            DB::beginTransaction();

            $Nutrition->fill($request->all());

            if ($request->has("Image")) {
                // Extract the image extension from the base64 string
                $extension = explode('/', mime_content_type($request->input("Image")))[1];

                // Decode the base64 image string
                $imageData = base64_decode(preg_replace('#^data:image/\w+;base64,#i', '', $request->input("Image")));

                // Generate a unique filename based on the current timestamp
                $imageName = time() . '_' . Str::random(10) . '.' . $extension;

                Storage::disk('public')->put('NutritionImages/' . $imageName, $imageData);


                $Nutrition->Image = 'NutritionImages/' . $imageName;

                // Delete the existing image from the media library
                if ($Nutrition->getFirstMedia("NutritionImages")) {
                    $Nutrition->getFirstMedia("NutritionImages")->forceDelete();
                }

                $Nutrition
                ->addMedia(storage_path('app/public/NutritionImages/' . $imageName))
                ->usingFileName($imageName)
                ->toMediaCollection("NutritionImages" , "NutritionImages");
            }

            $Nutrition->save();

            DB::commit();

            return response()->json(["message" => "Nutrition updated successfully ", "status" => 200]);

        }  catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {

        try {
            $nutrition = Nutrition::findOrFail($id);

            DB::beginTransaction();

            $nutrition->forceDelete();

            DB::commit();

            return response()->json(["message" => "Nutrition deleted successfully", "status" => 200]);
        } catch (ModelNotFoundException $e) {
            DB::rollBack();
            return response()->json(["error" => "Nutrition not found", "status" => 404]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }
}
