<?php

namespace App\Http\Controllers\API_MOBILE\NutritionTraceability;

use App\Http\Controllers\Controller;
use App\Http\Resources\NutritionTraceabilityResource;
use App\Models\Entreprise;
use App\Models\Nutrition;
use App\Models\NutritionTraceability;
use Exception;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\ValidationException;

class NutritionTraceabilityController_MOBILE extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        try {
            $NutritionTracability = NutritionTraceability::paginate(20);
            return NutritionTraceabilityResource::collection($NutritionTracability);
    }catch(Exception $e){
        return response()->json(["error" => $e->getMessage(), "status" => 500]);
    }
    }
    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        try {

            $validator = Validator::make($request->all(), [
                "EntrepriseICE" => "required|integer|exists:entreprises,ICE",
                "NutritionBarCode"=>"required|integer|exists:nutritions,BarCode",
                "UserId"=>"required|integer|exists:users,id",
                "Unite" => "required|string|in:kg,L,ml,g",
                "Quantite" =>"required|numeric",
                "NonConformities" =>"required|integer|in:0,1",
                "Raison"=>"string|max:255"
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            $entreprise = Entreprise::where('ICE', $request->input('EntrepriseICE'))->first();
            $nutrition = Nutrition::where("BarCode",$request->input("NutritionBarCode"))->first();

            if (!$entreprise) {
                throw new Exception('Enterprise not found');
            }

            DB::beginTransaction();

            $NutritionbTracability = new NutritionTraceability();
            $NutritionbTracability->EntrepriseId = $entreprise->id ;
            $NutritionbTracability->UserId = $request->input("UserId") ;
            $NutritionbTracability->NutritionId = $nutrition->id ;
            $NutritionbTracability->Unite = $request->input("Unite") ;
            $NutritionbTracability->Quantite = $request->input("Quantite");
            $NutritionbTracability->NonConformities = $request->input("NonConformities");
            $NutritionbTracability->Raison = $request->input("Raison");
            $NutritionbTracability->save();

            DB::commit();

            return response()->json(["message" => "Nutrition Tracability created successfully!", "status" => 201]);
        } catch (ValidationException $e) {
            DB::rollBack();
            return response()->json(["errors" => $e->errors(), "status" => 422]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {

        try {
            $entreprise = Entreprise::where('id', $id)->first();

            if (!$entreprise) {
                throw new Exception('Enterprise not found');
            }

            $NutritionTracability = NutritionTraceability::where("EntrepriseId",$entreprise->id)->with(["entreprise:id,name,ICE","nutrition:id,name","user:id,name"])->paginate(20);
            return NutritionTraceabilityResource::collection($NutritionTracability);

        } catch (ModelNotFoundException $e) {
            return response()->json(["error" => "Nutrition Tracability not found", "status" => 404]);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
        try {

            $NutritionTraceability = NutritionTraceability::findOrFail($id);

            $validator = Validator::make($request->all(), [
                "UserId"=>"required|integer|exists:users,id",
                "Unite" => "required|string|in:kg,L,ml,g",
                "Quantite" =>"required|numeric",
                "NonConformities" =>"required|integer|in:0,1",
                "Raison"=>"string|max:255"
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            DB::beginTransaction();

            $NutritionTraceability->fill($request->all());
            $NutritionTraceability->save();

            DB::commit();

            return response()->json(["message" => "Nutrition Tracability updated successfully", "status" => 200]);
        } catch (ValidationException $e) {
            DB::rollBack();
            return response()->json(["errors" => $e->errors(), "status" => 422]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        try {

            $NutritionTraceability = NutritionTraceability::findOrFail($id);

            DB::beginTransaction();

            $NutritionTraceability->forceDelete();

            DB::commit();

            return response()->json(["message" => "Nutrition Tracability  deleted successfully", "status" => 200]);
        } catch (ModelNotFoundException $e) {
            DB::rollBack();
            return response()->json(["error" => "Nutrition Tracability not found", "status" => 404]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }
}
