<?php

namespace App\Http\Controllers\API_MOBILE\Tache;


use App\Http\Controllers\Controller;
use App\Http\Resources\TacheResource;
use App\Models\Entreprise;
use App\Models\Tache;
use Exception;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\ValidationException;

class TacheController_MOBILE extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        try {
            $tache = Tache::paginate(20);
            return TacheResource::collection($tache);
        }catch (Exception $e){
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }

    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        try {


            $validator = Validator::make($request->all(), [
                "Name" => "required|string",
                "Date"=>"required|date",
                "Type"=>"required|string",
                "Category"=>"required|string",
                "Status"=>"string",
                "Message"=>"string|max:255",
                "EntrepriseICE" => "required|integer|exists:entreprises,ICE",
                "ZoneId"=>"required|integer|exists:zones,id",
                "UserId"=>"required|integer|exists:users,id",
                "Description" =>"string|max:255"
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            $entreprise = Entreprise::where('ICE', $request->input('EntrepriseICE'))->first();

            if (!$entreprise) {
                throw new ModelNotFoundException('Enterprise not found');
            }

            DB::beginTransaction();

            $tache = new Tache();
            $tache->Name = $request->input('Name');
            $tache->Date = $request->input("Date");
            $tache->Type = $request->input("Type") ;
            $tache->ZoneId = $request->input("ZoneId");
            $tache->UserId = $request->input("UserId");
            $tache->EntrepriseId = $entreprise->id ;
            $tache->Category = $request->input("Category");
            $tache->Status = $request->input("Status");
            $tache->Message = $request->input("Message") ;
            $tache->Description = $request->input("Description");
            $tache->EntrepriseId = $entreprise->id ;
            $tache->save();

            DB::commit();

            return response()->json(["message" => "Tache created successfully!", "status" => 201]);
        } catch (ValidationException $e) {
            DB::rollBack();
            return response()->json(["errors" => $e->errors(), "status" => 422]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        try {



            $entreprise = Entreprise::where('id', $id)->first();

            if (!$entreprise) {
                throw new Exception('Enterprise not found');
            }
            $tache = Tache::where("EntrepriseId",$entreprise->id)->with(["entreprise:id,name,ICE","zone:id,name","user:id,name"])->paginate(20);
            return TacheResource::collection($tache);

        } catch (ModelNotFoundException $e) {
            return response()->json(["error" => "Tache not found", "status" => 404]);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
        try {


            $tache = Tache::findOrFail($id);

            $validator = Validator::make($request->all(), [
                "Name" =>"string|max:255",
                "Date"=>"date",
                "Type"=>"string|max:255",
                "Category"=>"string|max:255",
                "Status"=>"string|max:255",
                "Message"=>"string|max:255",
                "ZoneId"=>"integer|exists:zones,id",
                "UserId"=>"integer|exists:users,id",
                "Description" =>"string|max:255"
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            DB::beginTransaction();

            $tache->fill($request->all());
            $tache->save();

            DB::commit();

            return response()->json(["message" => "Tache updated successfully", "status" => 200]);
        } catch (ValidationException $e) {
            DB::rollBack();
            return response()->json(["errors" => $e->errors(), "status" => 422]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }
    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        try {


            $tache = Tache::findOrFail($id);

            DB::beginTransaction();

            $tache->forceDelete();

            DB::commit();

            return response()->json(["message" => "Tache deleted successfully", "status" => 200]);
        } catch (ModelNotFoundException $e) {
            DB::rollBack();
            return response()->json(["error" => "Tache not found", "status" => 404]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }
}
