<?php

namespace App\Http\Controllers\API_MOBILE\Zone;

use App\Http\Controllers\Controller;
use App\Http\Resources\ZoneResource;
use App\Models\Entreprise;
use App\Models\Zone;
use Exception;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\ValidationException;

class ZoneController_MOBILE extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {

        try {
            $zone = Zone::paginate(20);
            return ZoneResource::collection($zone);
    }catch(Exception $e){
        return response()->json(["error" => $e->getMessage(), "status" => 500]);
    }

    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        try {

            $validator = Validator::make($request->all(), [
                "Name" => "required|string",
                "EntrepriseICE" => "required|integer",
                "Description" =>"string|max:255"
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            $entreprise = Entreprise::where('ICE', $request->input('EntrepriseICE'))->first();

            if (!$entreprise) {
                throw new Exception('Enterprise not found');
            }

            DB::beginTransaction();

            $zone = new Zone();
            $zone->Name = $request->input('Name');
            $zone->Description = $request->input("Description");
            $zone->EntrepriseId = $entreprise->id ;
            $zone->save();

            DB::commit();

            return response()->json(["message" => "Zone created successfully!", "status" => 201]);
        } catch (ValidationException $e) {
            DB::rollBack();
            return response()->json(["errors" => $e->errors(), "status" => 422]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        //dd($id);
        try {
            $zone = Zone::where("EntrepriseId",$id)->paginate(20);
            return ZoneResource::collection($zone);
        } catch (ModelNotFoundException $e) {
            return response()->json(["error" => "Zone not found", "status" => 404]);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
        try {

            $zone = Zone::findOrFail($id);

            $validator = Validator::make($request->all(), [
                "Name" => "string",
                "Description" => "string|max:255"
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            DB::beginTransaction();

            $zone->fill($request->all());
            $zone->save();

            DB::commit();

            return response()->json(["message" => "Zone updated successfully", "status" => 200]);
        } catch (ValidationException $e) {
            DB::rollBack();
            return response()->json(["errors" => $e->errors(), "status" => 422]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        try {




            $zone = Zone::findOrFail($id);

            DB::beginTransaction();

            $zone->forceDelete();

            DB::commit();

            return response()->json(["message" => "Zone deleted successfully", "status" => 200]);
        } catch (ModelNotFoundException $e) {
            DB::rollBack();
            return response()->json(["error" => "Zone not found", "status" => 404]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }
}
