<?php

namespace App\Http\Controllers\API_WEB\Client;

use App\Http\Controllers\Controller;
use App\Http\Resources\ClientResource;
use App\Models\Client;
use App\Models\Entreprise;
use Exception;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\ValidationException;

class ClientController_WEB extends Controller
{
    public function index()
    {

        try {
            $Client = Client::with("entreprise:id,name,ICE")->paginate(20);
            return ClientResource::collection($Client);
    }catch(Exception $e){
        return response()->json(["error" => $e->getMessage(), "status" => 500]);
    }

    }
    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                "Name"=>"string|max:255",
                "Pays"=>"string|max:255",
                "Ville"=>"string|max:255",
                "Telephone"=>"string|max:20",
                "Adresse"=>"string|max:255",
                "EntrepriseId"=>"required|integer|exists:entreprises,id"
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }
            DB::beginTransaction();

            $Client = new Client();
            $Client->Name = $request->input('Name');
            $Client->Adresse = $request->input('Adresse');
            $Client->Ville = $request->input("Ville");
            $Client->Telephone = $request->input("Telephone");
            $Client->Pays = $request->input("Pays");
            $Client->EntrepriseId = $request->input("EntrepriseId") ;

            $Client->save();

            DB::commit();

            return response()->json(["message" => "Client created successfully!", "status" => 201]);
        } catch (ValidationException $e) {
            DB::rollBack();
            return response()->json(["errors" => $e->errors(), "status" => 422]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        try {
            $Client = Client::where("EntrepriseId",$id)->with("entreprise:id,name,ICE")->paginate(20);
            return ClientResource::collection($Client);
    }catch(Exception $e){
        return response()->json(["error" => $e->getMessage(), "status" => 500]);
    }
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        //

    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
        try {

            $Client = Client::findOrFail($id);

            $validator = Validator::make($request->all(), [
                "Name"=>"string|max:255",
                "Pays"=>"string|max:255",
                "Ville"=>"string|max:255",
                "Telephone"=>"string|max:20",
                "Adresse"=>"string|max:255",
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            DB::beginTransaction();

            $Client->fill($request->all());
            $Client->save();

            DB::commit();

            return response()->json(["message" => "Client updated successfully", "status" => 200]);
        } catch (ValidationException $e) {
            DB::rollBack();
            return response()->json(["errors" => $e->errors(), "status" => 422]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        try {
            $client = Client::findOrFail($id);

            DB::beginTransaction();

            $client->forceDelete();

            DB::commit();

            return response()->json(["message" => "Client deleted successfully", "status" => 200]);
        } catch (ModelNotFoundException $e) {
            DB::rollBack();
            return response()->json(["error" => "Client not found", "status" => 404]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

}
