<?php

namespace App\Http\Controllers\API_WEB\Equipement;

use App\Http\Controllers\Controller;
use App\Http\Resources\EquipementResource;
use App\Models\Entreprise;
use App\Models\Equipement;
use Exception;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\ValidationException;

class EquipementController_WEB extends Controller
{
    public function index()
    {
        try {
            $Equipement = Equipement::with(["entreprise:id,name,ICE","zone:id,name"])->paginate(20);
            return EquipementResource::collection($Equipement);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {

        try {

            $validator = Validator::make($request->all(), [
                "Name" => "required|string",
                "EntrepriseId" => "required|integer|exists:entreprises,id",
                "ZoneId"=>"required|integer|exists:zones,id" ,
                "Description"=>"string|max:255"
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            DB::beginTransaction();

            $Equipement = new Equipement();
            $Equipement->Name = $request->input('Name');
            $Equipement->EntrepriseId = $request->input('EntrepriseId') ;
            $Equipement->ZoneId = $request->input("ZoneId") ;
            $Equipement->Description = $request->input("Description");
            $Equipement->save();

            DB::commit();

            return response()->json(["message" => "Equipement created successfully!", "status" => 201]);
        } catch (ValidationException $e) {
            DB::rollBack();
            return response()->json(["errors" => $e->errors(), "status" => 422]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }
    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        try {

            $Equipement = Equipement::where("EntrepriseId",$id)->with(["entreprise:id,name,ICE","zone:id,name"])->paginate(20);
            return EquipementResource::collection($Equipement);
        } catch (ModelNotFoundException $e) {
            return response()->json(["error" => "Equipement not found", "status" => 404]);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }


    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
        try {



            $equipement = Equipement::findOrFail($id);

            $validator = Validator::make($request->all(), [
                "Name" => "string|max:255",
                "ZoneId"=>"integer",
                "Description"=>"string|max:255"
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            DB::beginTransaction();

            $equipement->fill($request->all());
            $equipement->save();

            DB::commit();

            return response()->json(["message" => "Equipement updated successfully", "status" => 200]);
        } catch (ValidationException $e) {
            DB::rollBack();
            return response()->json(["errors" => $e->errors(), "status" => 422]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {

        try {

            $equipement = Equipement::findOrFail($id);

            DB::beginTransaction();

            $equipement->forceDelete();

            DB::commit();

            return response()->json(["message" => "Equipement deleted successfully", "status" => 200]);
        } catch (ModelNotFoundException $e) {
            DB::rollBack();
            return response()->json(["error" => "Equipement not found", "status" => 404]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

}
