<?php

namespace App\Http\Controllers\API_WEB\Fournisseur;

use App\Http\Controllers\Controller;
use App\Http\Resources\EquipementResource;
use App\Http\Resources\FournisseurResource;
use App\Models\Fournisseur;
use Exception;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\ValidationException;
use Illuminate\Support\Str;


class FournisseurController_WEB extends Controller
{



    public function allFournisseur(){
        try {
            $Fournisseur = Fournisseur::with("entreprise:id,name,ICE")->get();

            return FournisseurResource::collection($Fournisseur);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }



    public function index()
    {

        try {
            $Fournisseur = Fournisseur::with("entreprise:id,name,ICE")->paginate(20);
            return FournisseurResource::collection($Fournisseur);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }

    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                "Name" => "required|string",
                "Image" => "nullable|string", // Assuming the image is sent as base64 string
                "EntrepriseId" =>"required|integer|exists:entreprises,id",
                "Email" => "required|email|unique:fournisseurs,email|max:255",
                "Telephone" => "required|string|max:20",
                "Adresse" => "required|string|max:255",
                "Pays" => "required|string|max:255",
                "Ville" => "required|string|max:255"
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            DB::beginTransaction();

            $fournisseur = new Fournisseur();
            $fournisseur->Name = $request->input('Name');
            $fournisseur->EntrepriseId = $request->input('EntrepriseId');
            $fournisseur->Email = $request->input('Email');
            $fournisseur->Telephone = $request->input('Telephone');
            $fournisseur->Adresse = $request->input('Adresse');
            $fournisseur->Pays = $request->input('Pays');
            $fournisseur->Ville = $request->input('Ville');

            if ($request->has("Image")) {
                // Extract the image extension from the base64 string
                $extension = explode('/', mime_content_type($request->input("Image")))[1];

                // Decode the base64 image string
                $imageData = base64_decode(preg_replace('#^data:image/\w+;base64,#i', '', $request->input("Image")));

                // Generate a unique filename based on the current timestamp
                $imageName = time() . '_' . Str::random(10) . '.' . $extension;


                Storage::disk('public')->put('FournisseurImages/' . $imageName, $imageData);


                $fournisseur->Image = 'FournisseurImages/' . $imageName;


                $fournisseur
                ->addMedia(storage_path('app/public/FournisseurImages/' . $imageName))
                ->usingFileName($imageName)
                ->toMediaCollection("FournisseurImages" , "FournisseurImages");
            }

            $fournisseur->save();

            DB::commit();

            return response()->json(["message" => "Fournisseur created successfully!", "status" => 201]);
        } catch (ValidationException $e) {
            DB::rollBack();
            return response()->json(["errors" => $e->errors(), "status" => 422]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {

        try {

            $fournisseur = Fournisseur::where("EntrepriseId",$id)->with("entreprise:id,name,ICE")->get();
            return FournisseurResource::collection($fournisseur);
        } catch (ModelNotFoundException $e) {
            return response()->json(["error" => "Fournisseur not found", "status" => 404]);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
        try {

            $fournisseur = Fournisseur::findOrFail($id);

            $validator = Validator::make($request->all(),[
                "Name" => "string",
                "Image" => "nullable|string", // Assuming the image is sent as base64 string
                "Email" => "email|unique:fournisseurs,email|max:255",
                "Telephone" => "string|max:20",
                "Adresse" => "string|max:255",
                "Pays" => "string|max:255",
                "Ville" => "string|max:255",
                //"EnterpriseId"=>"integer|unique:fournisseurs,email|max:255"
            ]);

            if ($validator->fails()) {
                throw ValidationException::withMessages($validator->errors()->toArray());
            }

            DB::beginTransaction();



            $fournisseur->fill($request->all());

            if ($request->has("Image")) {
                // Extract the image extension from the base64 string
                $extension = explode('/', mime_content_type($request->input("Image")))[1];

                // Decode the base64 image string
                $imageData = base64_decode(preg_replace('#^data:image/\w+;base64,#i', '', $request->input("Image")));

                // Generate a unique filename based on the current timestamp
                $imageName = time() . '_' . Str::random(10) . '.' . $extension;

                Storage::disk('public')->put('FournisseurImages/' . $imageName, $imageData);


                $fournisseur->Image = 'FournisseurImages/' . $imageName;

                // Delete the existing image from the media librarye
                if ($fournisseur->getFirstMedia("FournisseurImages")) {
                    $fournisseur->getFirstMedia("FournisseurImages")->forceDelete();
                }

                $fournisseur
                ->addMedia(storage_path('app/public/FournisseurImages/' . $imageName))
                ->usingFileName($imageName)
                ->toMediaCollection("FournisseurImages" , "FournisseurImages");
            }

            $fournisseur->save();

            DB::commit();

            return response()->json(["message" => "Fournisseur updated successfully ", "status" => 200]);

        }  catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }

    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        try {

            $fournisseur = Fournisseur::findOrFail($id);

            DB::beginTransaction();

            // Delete the image associated with the enterprise from the media library
            if ($fournisseur->getFirstMedia("FournisseurImages")) {
                $fournisseur->getFirstMedia("FournisseurImages")->forceDelete();
            }

            // Delete the enterprise
            $fournisseur->forceDelete();

            DB::commit();

            return response()->json(["message" => "Fournisseur deleted successfully!", "status" => 200]);
        } catch (ModelNotFoundException $e) {
            DB::rollBack();
            return response()->json(["error" => "Fournisseur not found", "status" => 404]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }



}
