<?php

namespace App\Http\Controllers\API_WEB\User;

use App\Http\Controllers\Controller;
use App\Http\Resources\UserResource;
use App\Models\User;
use Exception;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

class UserController_WEB extends Controller
{
    public function index()
    {
        try {
            $User = User::paginate(20);
            return UserResource::collection($User);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
           try{
            $validated = $request->validate([
                "name"=>"required|string",
                "entrepriseId"=>"required|integer|exists:entreprises,id",
                "email"=>"nullable|email",
                "telephone"=>"nullable|string",
                "adresse"=>"nullable|string",
                "ville"=>"nullable|string",
                "Image"=>"nullable|string",
                "password"=>"required|string",
            ]);

            $user = new User();
            $user->name = $validated["name"];
            $user->entrepriseId = $validated["entrepriseId"];
            $user->email = $validated["email"];
            $user->Telephone = $validated["telephone"];
            $user->Adresse = $validated["adresse"];
            $user->Ville = $validated["ville"];
            $user->password = bcrypt($validated["password"]);


            if ($request->has("Image")) {
                // Extract the image extension from the base64 string
                $extension = explode('/', mime_content_type($validated['Image']))[1];

                // Decode the base64 image string
                $imageData = base64_decode(preg_replace('#^data:image/\w+;base64,#i', '', $validated['Image']));

                // Generate a unique filename based on the current timestamp
                $imageName = time() . '_' . Str::random(10) . '.' . $extension;

                Storage::disk('public')->put('UserImages/' . $imageName, $imageData);


                $user->Image = 'UserImages/' . $imageName;

                $user->addMedia(storage_path('app/public/UserImages/' . $imageName))->usingFileName($imageName)->toMediaCollection("UserImages" , "UserImages");

                $user->save();

               return response()->json(["message" => "User created successfully", "status" => 201]);
            }
        }catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        try {


            $user = User::where("EntrepriseId",$id)->paginate(20);
            return UserResource::collection($user);
        } catch (ModelNotFoundException $e) {
            return response()->json(["error" => "User not found", "status" => 404]);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

    public function ShowByUserId(string $id){

        try {
            $user = User::findOrFail($id)->get();
           return  UserResource::collection($user);
        } catch (ModelNotFoundException $e) {
            return response()->json(["error" => "User not found", "status" => 404]);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);

        }
    }


    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        //
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, string $id)
    {
         try {
            $validated = $request->validate([
                "name" =>"nullable|string",
                "entrepriseId"=>"nullable|integer|exists:entreprises,id",
                "email"=>"nullable|email",
                "telephone"=>"nullable|string",
                "adresse"=>"nullable|string",
                "ville"=>"nullable|string",
                "Image"=>"nullable|string",
                "password"=>"nullable|string",
            ]);

            $user = User::findOrFail($id);
            $user->name = $validated["name"];
            $user->entrepriseId = $validated["entrepriseId"];
            $user->email = $validated["email"];
            $user->telephone = $validated["telephone"];
            $user->adresse = $validated["adresse"];
            $user->ville = $validated["ville"];
            $user->password = bcrypt($validated["password"]);

            if ($request->has("Image")) {
                // Extract the image extension from the base64 string
                $extension = explode('/', mime_content_type($validated['Image']))[1];

                // Decode the base64 image string
                $imageData = base64_decode(preg_replace('#^data:image/\w+;base64,#i', '', $validated['Image']));

                // Generate a unique filename based on the current timestamp
                $imageName = time() . '_' . Str::random(10) . '.' . $extension;

                Storage::disk('public')->put('UserImages/' . $imageName, $imageData);


                $user->Image = 'UserImages/' . $imageName;

                // Delete the existing image from the media library
                if ($user->getFirstMedia("UserImages")) {
                    $user->getFirstMedia("UserImages")->forceDelete();
                }

                $user->addMedia(storage_path('app/public/UserImages/' . $imageName))->usingFileName($imageName)->toMediaCollection("UserImages" , "UserImages");

                $user->save();

               return response()->json(["message" => "User Updated Successfully", "status" => 201]);
            }


         }catch (Exception $e) {
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
    }}

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        try {


            $user = User::findOrFail($id);
            DB::beginTransaction();

            $user->forceDelete();

            DB::commit();

            return response()->json(["message" => "User deleted successfully", "status" => 200]);
        } catch (ModelNotFoundException $e) {
            DB::rollBack();
            return response()->json(["error" => "User not found", "status" => 404]);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json(["error" => $e->getMessage(), "status" => 500]);
        }
    }

}
