<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Mail\ForgotPasswordMail;
use App\Models\User;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Mail;
use Laravel\Passport\PersonalAccessTokenFactory;

class ForgotPasswordController extends Controller
{
    public function GetUserInfo(Request $request)
    {
        try {
            $request->validate(['email' => 'required|email']);

            $user = User::where('email', $request->email)->first();

            if (!$user) {
                throw new Exception('User Not Found !');
            }

            // Create a new personal access token for the user
            $tokenResult = $user->createToken('password_reset');
            $token = $tokenResult->accessToken;
            $resetLink = 'http://localhost:5173/reset-password?user=' . urlencode($user->email) . '&token=' . $token;

            // Send reset password email
            Mail::to($user->email)->send(new ForgotPasswordMail($resetLink, $token));

            return response()->json([
                "User" => $user,
                "status" => 200
            ]);
        } catch (Exception $e) {
            return response()->json(['error' => $e->getMessage(), "status" => 500]);
        }
    }
}

