<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Http\Resources\UserResource;
use App\Models\Entreprise;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;

class RegisterController extends Controller
{
    public function register(Request $request){
        try {
            // Validation rules
            $validator = Validator::make($request->all(), [
                'name' => 'required|string|max:255',
                'email' => 'required|email|unique:users,email',
                'password' => 'required|string|min:8',
                "EntrepriseICE"=>"required|integer|exists:entreprises,ICE"
            ]);

            // Check if validation fails
            if ($validator->fails()) {
                return response()->json(["success"=>false,'error' => $validator->errors() , "status"=>400], );
            }

            $entreprise = Entreprise::where("ICE", $request->EntrepriseICE)->first();

            if (!$entreprise) {
                return response()->json(["success"=>false,'error' => 'The provided EntrepriseICE does not exist.', "status"=>400]);
            }


            $user = new User();
            $user->name = $request->name;
            $user->email = $request->email;
            $user->password = Hash::make($request->password);
            $user->EntrepriseId = $entreprise->id;
            $user->save();


            $user = new UserResource($user);

            return response()->json([ "user"=>$user, "success"=>true , "Message"=>"Successfully Registered !! ","status"=>201], );
        } catch (\Exception $e) {

            // Return error response
            return response()->json(["success"=>false,'error' => 'User registration failed. Please try again later !!' , "Message"=>$e->getMessage(),"status"=>500] );
        }
    }
    }

