<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\ValidationException;
use Illuminate\Database\Eloquent\ModelNotFoundException;

class ResetPasswordController extends Controller
{
   public function ResetPassword(Request $request)
   {
        try {
            $validator = Validator::make($request->all(), [
                'email' => 'required|email|exists:users,email',
                'password' => 'required|string|min:8',
                'ConfirmPassword' => 'required|same:password',
            ]);

            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            $user = User::where('email', $request->email)->first();

            $user->password = bcrypt($request->password);
            $user->save();

            $user->tokens()->delete();

            $token = $user->createToken('Reset_Password_Access_Token')->accessToken;

            return response()->json([
                'message' => 'Password reset successfully!',
                'token' => $token,
            ], 200);
        } catch (ValidationException $e) {
            return response()->json([
                'error' => $e->errors(),
                "status"=>400
            ]);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'error' => 'User not found!',
                "status"=>404
            ]);
        } catch (\Exception $e) {
            return response()->json([
                'error' => 'Failed to reset password!',
                "status"=>500
            ]);
        }
   }
}
