<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\SoftDeletes;

class Equipement extends Model
{
    use HasFactory , SoftDeletes;

    /**
     * The table associated with the model.
     *
     * @var string
     */
    protected $table = 'equipements';

    /**
     * The primary key associated with the table.
     *
     * @var string
     */
    protected $primaryKey = 'id';

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'Name',
        'ZoneId',
        'EntrepriseId',
        'Description',
    ];

    /**
     * Get the zone that owns the equipment.
     */
    public function zone() : BelongsTo
    {
        return $this->belongsTo(Zone::class, 'ZoneId');
    }

    /**
     * Get the enterprise that owns the equipment.
     */
    public function entreprise() : BelongsTo
    {
        return $this->belongsTo(Entreprise::class, 'EntrepriseId');
    }
}
