<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\SoftDeletes;
use Spatie\MediaLibrary\HasMedia;
use Spatie\MediaLibrary\InteractsWithMedia;

class Nutrition extends Model implements HasMedia
{
    use HasFactory , InteractsWithMedia , SoftDeletes;


     /**
     * The table associated with the model.
     *
     * @var string
     */
    protected $table = 'nutritions';

    /**
     * The primary key associated with the table.
     *
     * @var string
     */
    protected $primaryKey = 'id';

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'ChambreId',
        'EntrepriseId',
        'FournisseurId',
        'Image',
        'BarCode',
        'Name',
        'DLCDate',
        'DDMDate',
        'ExpirationAlert',
        'Category',
        'Description',
    ];

    /**
     * Get the chambre that owns the nutrition.
     */
    public function chambre() : BelongsTo
    {
        return $this->belongsTo(Chambre::class, 'ChambreId');
    }

    /**
     * Get the enterprise that owns the nutrition.
     */
    public function entreprise() : BelongsTo
    {
        return $this->belongsTo(Entreprise::class, 'EntrepriseId');
    }

    /**
     * Get the fournisseur that owns the nutrition.
     */
    public function fournisseur() : BelongsTo
    {
        return $this->belongsTo(Fournisseur::class, 'FournisseurId');
    }
}
