<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\SoftDeletes;

class NutritionTraceability extends Model
{
    use HasFactory , SoftDeletes;

     /**
     * The table associated with the model.
     *
     * @var string
     */
    protected $table = 'nutrition_traceabilities';

    /**
     * The primary key associated with the table.
     *
     * @var string
     */
    protected $primaryKey = 'id';

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'CommandeId',
        'EntrepriseId',
        'NutritionId',
        'UserId',
        'Unite',
        'Quantite',
        'NonConformities',
        'Raison',
    ];

    /**
     * Get the commande that owns the nutrition traceability.
     */
    public function commande() : BelongsTo
    {
        return $this->belongsTo(Commande::class, 'CommandeId');
    }

    /**
     * Get the entreprise that owns the nutrition traceability.
     */
    public function entreprise() : BelongsTo
    {
        return $this->belongsTo(Entreprise::class, 'EntrepriseId');
    }

    /**
     * Get the nutrition that owns the nutrition traceability.
     */
    public function nutrition() : BelongsTo
    {
        return $this->belongsTo(Nutrition::class, 'NutritionId');
    }

    /**
     * Get the user that owns the nutrition traceability.
     */
    public function user() : BelongsTo
    {
        return $this->belongsTo(User::class, 'UserId');
    }
}
